<?php
namespace Findologic\Search\Helper;

use Findologic\Search\FindologicCustomExportOrig;
use Magento\Framework\App\ObjectManager;
use Magento\Catalog\Model\Product;
use Magento\Catalog\Model\ResourceModel\Product\Collection;

class Xml
{
    /**
     * @var string
     */
    private $shopKey;

    /**
     * Builds xml document.
     *
     * @param array $params
     * @param Collection $products
     * @return string Xml file
     */
    public function buildXmlDoc($params, $products)
    {
        $this->shopKey = $params['shopkey'];

        $findologic = new \SimpleXMLElement('<findologic/>');
        $findologic->addAttribute('version', '1.0');
        $items = $findologic->addChild('items');
        $items->addAttribute('start', $params['start']);
        $items->addAttribute('count', $params['count']);
        $items->addAttribute('total', $params['total']);

        $om = ObjectManager::getInstance();

        $xmlBasic = class_exists('Findologic\Search\FindologicCustomExport')
            ? $om->get('\Findologic\Search\FindologicCustomExport')
            : $om->get('\Findologic\Search\Helper\XmlBasic');

        $helper = $om->get('\Findologic\Search\Helper\TrackingScripts');

        /** @var $product \Magento\Catalog\Model\Product */
        foreach ($products as $product) {
            $item = $items->addChild('item');
            $item->addAttribute('id', $product->getId());
            $xmlBasic->setData($params, $product);
            $this->renderItem($item, $helper, $xmlBasic);
        }

        return $findologic->asXML();
    }

    /**
     * Renders one product.
     *
     * @param \SimpleXMLElement $item
     * @param TrackingScripts $helper
     * @param XmlBasic $xmlBasic
     */
    private function renderItem(\SimpleXMLElement $item, TrackingScripts $helper, XmlBasic $xmlBasic)
    {
        $xmlBasic->renderOrderNumber($item->addChild('allOrdernumbers'));

        $xmlBasic->renderName($item->addChild('names'));

        $xmlBasic->renderSummary($item->addChild('summaries'));

        $xmlBasic->renderDescription($item->addChild('descriptions'));

        $xmlBasic->renderPrice($item->addChild('prices'));

        $xmlBasic->renderUrl($item->addChild('urls'));

        $xmlBasic->renderImages($item->addChild('allImages'));

        $xmlBasic->renderAttributes($item->addChild('allAttributes'));

        $xmlBasic->renderKeywords($item->addChild('allKeywords'));

        // By default product visibility can be set only for different stores, not for user group, so we render all groups.
        $userGroups = $item->addChild('usergroups');
        $om = ObjectManager::getInstance();
        $groups = $om->create('Magento\Customer\Model\Group')->getCollection();

        /** @var  $group \Magento\Customer\Model\Group */
        foreach($groups as $group) {
            if ($group->getId() > 0) {
                $this->appendCData($userGroups->addChild('usergroup'),
                    $helper->userGroupToHash($this->shopKey, $group->getCustomerGroupCode())
                );
            }
        }

        $xmlBasic->renderSalesFrequency($item->addChild('salesFrequencies'));

        $xmlBasic->renderDateAdded($item->addChild('dateAddeds'));

        $xmlBasic->renderAllProperties($item->addChild('allProperties'));
    }
    
    private function appendCData(\SimpleXMLElement $node, $text)
    {
        /** @var \DOMElement $domNode */
        $domNode = dom_import_simplexml($node);
        $domNode->appendChild($domNode->ownerDocument->createCDATASection($this->utf8Replace($text)));
    }

    private function utf8Replace($text)
    {
        return preg_replace('/[\x00-\x08\x0B\x0C\x0E-\x1F\x80-\x9F]/u', '', trim($text));
    }
}
