<?php
namespace Findologic\Search\Helper;

use Findologic\Search\Model\DTO\PriceDto;
use Magento\Framework\App\DeploymentConfig;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Config\ConfigOptionsListConstants;
use Magento\Framework\UrlInterface;
use Magento\Catalog\Model\Product;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\Catalog\Model\ProductFactory;
use Findologic\Search\Model\Price;
use \Magento\Framework\App\Config\ScopeConfigInterface;

class XmlBasic extends AbstractHelper
{
    /**
     * @var string
     */
    protected $shopKey;

    /**
     * @var int
     */
    protected $storeId;

    /**
     * @var int
     */
    protected $rootCategoryId;

    /**
     * @var Product
     */
    protected $product;

    /**
     * @var array[Product]
     */
    protected $variations;

    /**
     * @var PriceDto
     *
     */
    protected $productPrice;

    /**
     * @var ProductFactory
     */
    protected $productFactory;

    /**
     * @var DeploymentConfig
     */
    protected $deploymentConfig;

    /**
     * @var ScopeConfigInterface
     */
    protected $storeConfig;

    protected static $customerGroups = array();
    protected static $baseUrl;
    protected static $mediaHelper;

    /**
     * @param Context $context
     * @param DeploymentConfig $deploymentConfig
     * @param ProductFactory $productFactory
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        Context $context,
        DeploymentConfig $deploymentConfig,
        ProductFactory $productFactory,
        ScopeConfigInterface $scopeConfig
    )
    {
        parent::__construct($context);
        $this->productFactory = $productFactory;
        $this->deploymentConfig = $deploymentConfig;
        $this->storeConfig = $scopeConfig;
    }

    /**
     * Static function that initializes all static fields to enhance performance
     */
    protected static function initStaticFields()
    {
        if (empty(self::$customerGroups)) {
            $om = ObjectManager::getInstance();
            /** @var $collection \Magento\Customer\Model\ResourceModel\Group\Collection */
            $collection = $om->get('\Magento\Customer\Model\ResourceModel\Group\Collection');
            /** @var  $group \Magento\Customer\Model\Group */
            foreach ($collection as $group) {
                self::$customerGroups[$group->getId()] = $group->getCustomerGroupCode();
            }
            self::$customerGroups[32000] = 'ALL GROUPS';
            self::$baseUrl = $om->get('\Magento\Store\Model\StoreManagerInterface')->getStore()
                ->getBaseUrl(UrlInterface::URL_TYPE_WEB);
            self::$mediaHelper = $om->get('\Magento\Catalog\Model\Product\Media\Config');
        }
    }

    /**
     * Sets data needed for all the methods. This method must be called before all other methods.
     *
     * @param $params
     * @param Product $product
     * @return $this
     */
    public function setData($params, Product $product)
    {
        self::initStaticFields();
        $this->shopKey = $params['shopkey'];
        $this->storeId = $params['storeId'];

        $om = ObjectManager::getInstance();

        $this->rootCategoryId =$om->get('Magento\Store\Model\StoreManagerInterface')->getStore($this->storeId)->getRootCategoryId();

        $this->product = $product;

        $this->variations = array();

        $this->productPrice = new Price($this->product);
        $this->productPrice = $this->productPrice->getProductPrice();

        if ($product && $product->getTypeId() === (Configurable::TYPE_CODE)) {
            $variations = $product->getTypeInstance(true)->getUsedProducts($product);
            $variationIds = array();
            /** @var $variation \Magento\Catalog\Model\Product */
            foreach ($variations as $variation) {
                if ($variation->isAvailable()) {
                    $variationIds[] = $variation->getId();
                    $this->variations[] = $variation;
                }
            }
        }

        return $this;
    }

    /**
     * Renders order number.
     *
     * @param \SimpleXMLElement $allOrderNumbers
     */
    public function renderOrderNumber(\SimpleXMLElement $allOrderNumbers)
    {
        $orderNumbers = $allOrderNumbers->addChild('ordernumbers');
        $this->appendCData($orderNumbers->addChild('ordernumber'), $this->product->getSku());
        /** @var $variation \Magento\Catalog\Model\Product */
        foreach ($this->variations as $variation) {
            if ($variation->isAvailable()) {
                $this->appendCData($orderNumbers->addChild('ordernumber'), $variation->getSku());
            }
        }
    }

    /**
     * Renders name.
     *
     * @param \SimpleXMLElement $names
     */
    public function renderName(\SimpleXMLElement $names)
    {
        $this->appendCData($names->addChild('name'), $this->product->getName());
    }

    /**
     * Renders summary (short description).
     *
     * @param \SimpleXMLElement $summaries
     */
    public function renderSummary(\SimpleXMLElement $summaries)
    {
        $this->appendCData($summaries->addChild('summary'), $this->product->getShortDescription());
    }

    /**
     * Renders long description.
     *
     * @param \SimpleXMLElement $descriptions
     */
    public function renderDescription(\SimpleXMLElement $descriptions)
    {
        $this->appendCData($descriptions->addChild('description'), $this->product->getDescription());
    }

    /**
     * Renders price.
     *
     * @param \SimpleXMLElement $prices
     */
    public function renderPrice(\SimpleXMLElement $prices)
    {
        $price = $this->productPrice->actualPrice;

        $groupPrices = $this->getGroupPrices($this->product);

        /** @var $variation \Magento\Catalog\Model\Product */
        foreach ($this->variations as $variation) {
            $price = min($price, ($variation->getSpecialPrice() ?: $variation->getPrice()));
            $groupPrices = array_merge_recursive($groupPrices, $this->getGroupPrices($variation));
        }

        $this->appendCData($prices->addChild('price'), sprintf('%.2f', $price));

        foreach ($groupPrices as $code => $priceArray) {
            $priceNode = $prices->addChild('price');
            $this->appendCData($priceNode, sprintf('%.2f', min($priceArray)));
            $priceNode->addAttribute('usergroup', $code);
        }
    }

    //TODO Investigate these group prices, and move them to respective model if needed.
    protected function getGroupPrices(Product $product)
    {
        /** @var \Findologic\Search\Helper\TrackingScripts $helper */
        $om = ObjectManager::getInstance();
        $helper = $om->get('\Findologic\Search\Helper\TrackingScripts');
        $attribute = $product->getResource()->getAttribute('tier_price');
        $attribute->getBackend()->afterLoad($this->product);
        $groupPrices = $product->getData('tier_price');

        $result = array();
        if ($groupPrices) {
            foreach ($groupPrices as $group) {
                $userGroup = self::$customerGroups[$group['cust_group']];
                $result[$helper->userGroupToHash($this->shopKey, $userGroup)] = array($group['price']);
            }
        }

        return $result;
    }

    /**
     * Render URL.
     *
     * @param \SimpleXMLElement $urls
     */
    public function renderUrl(\SimpleXMLElement $urls)
    {
        $this->appendCData($urls->addChild('url'), $this->product->getProductUrl());
    }

    /**
     * Renders a list of images, where the base image (if available) is the first one.
     * 
     * @param \SimpleXMLElement $allImages
     */
    public function renderImages(\SimpleXMLElement $allImages)
    {
        $product = $this->productFactory->create()->load($this->product->getId());
        $productImages = $product->getMediaGalleryImages();
        $productBaseImage = $product->getData('image');

        $images = $allImages->addChild('images');

        if ($productBaseImage !== 'no_selection') {
            $this->generateExportImage($images, $productBaseImage);
        }

        if ($productImages->count() > 0) {
            foreach ($productImages as $productImage) {
                if ($productImage->getFile() !== $productBaseImage) {
                    $this->generateExportImage($images, $productImage->getFile());
                }
            }
        } else {
            //If there are no images, render default small placeholder image
            $this->generateExportImage($images, $this->storeConfig->getValue('catalog/placeholder/small_image_placeholder'));
        }
    }

    /**
     * Renders all attributes (categories, categories url).
     *
     * @param \SimpleXMLElement $allAttributes
     */
    public function renderAttributes(\SimpleXMLElement $allAttributes)
    {
        $attributesNode = $allAttributes->addChild('attributes');
        $this->renderCategories($attributesNode);

        $this->addAttributes($attributesNode);

        $this->removeEmptyNode($attributesNode, $attributesNode);
    }

    /**
     * Renders categories and category urls as attributes.
     *
     * @param \SimpleXMLElement $attributesNode
     */
    protected function renderCategories(\SimpleXMLElement $attributesNode)
    {
        $catCollection = $this->getCategories();

        if (count($catCollection) === 0) {
            return;
        }

        $catNode = $attributesNode->addChild('attribute');

        $this->appendCData($catNode->addChild('key'), 'cat');
        $valuesNode = $catNode->addChild('values');

        $catUrlNode = $attributesNode->addChild('attribute');
        $this->appendCData($catUrlNode->addChild('key'), 'cat_url');
        $valuesCatUrl = $catUrlNode->addChild('values');

        $om = ObjectManager::getInstance();
        /** @var  $helper \Findologic\Search\Helper\TrackingScripts */
        $helper = $om->get('\Findologic\Search\Helper\TrackingScripts');

        /** @var $cat \Magento\Catalog\Model\Category */
        foreach ($catCollection as $cat) {
            $cat = $helper->getCategory($cat->getId());

            if ($cat->getIsActive()) {
                $catPath = $helper->getCategoryPath($cat->getPath(), '_', false, false);
                if ($catPath) {
                    $this->appendCData($valuesNode->addChild('value'), $catPath);
                    $catUrl = parse_url($cat->getUrl());
                    $valuesCatUrl->addChild('value', $catUrl['path']);
                }
            }
        }

        $this->removeEmptyNode($valuesNode, $catNode);
        $this->removeEmptyNode($valuesCatUrl, $catUrlNode);
    }

    /**
     * Filter categories by rootCategoryId.
     *
     * @return array $collection
     */
    protected function getCategories()
    {
        $om = ObjectManager::getInstance();
        $collection = $om->create('Magento\Catalog\Model\ResourceModel\Category\Collection')
            ->joinField('product_id',
                'catalog_category_product',
                'product_id',
                'category_id = entity_id',
                null)
            ->addFieldToFilter('product_id', (int)$this->product->getId())
            ->addFieldToFilter('path', array('like'=> "1/$this->rootCategoryId/%"));

        return $collection;
    }

    /**
     * Adds all configurable attributes as SimpleXMLElement array.
     *
     * @param \SimpleXMLElement $attributesNode
     */
    protected function addAttributes(\SimpleXMLElement $attributesNode)
    {
        $om = ObjectManager::getInstance();
        $result = array();
        $this->addAttributesForProduct($result, $this->product);

        foreach ($this->variations as $variation) {
            $this->addAttributesForProduct($result, $variation);
        }

        // Load attributes for variations.
        /** @var $attribute \Magento\ConfigurableProduct\Model\Product\Type\Configurable\Attribute */
        if (!empty($this->variations)) {
            $productAttributes = $this->product->getTypeInstance()->getConfigurableAttributes($this->product);
            foreach ($productAttributes as $attribute) {
                $attributeCode = $attribute->getProductAttribute()->getAttributeCode();
                $values = array();
                /** @var $variation \Magento\Catalog\Model\Product */
                foreach ($this->variations as $variation) {
                    if ($variation->isAvailable()) {
                        $attributeValue = trim($variation->getAttributeText($attributeCode));
                        if ($attributeValue && array_search($attributeValue, $values) === false) {
                            $values[] = $attributeValue;
                        }
                    }
                }

                if (count($values)) {
                    $result[$attributeCode] = $values;
                }
            }
        }

        $storeManager = $om->get('\Magento\Store\Model\StoreManagerInterface');
        $reviewFactory = $om->get('\Magento\Review\Model\ReviewFactory');
        $reviewFactory->create()->getEntitySummary($this->product, $storeManager->getStore()->getId());
        $ratingPercent = $this->product->getRatingSummary()->getRatingSummary();

        $rating = round($ratingPercent / 100 * 5);

        $result['votes_rating'] = array($rating);

        if ($this->product->getQty() < 10) {
            $result['availability'] = array("Only some items on stock - order quickly!");
        } else {
            $result['availability'] = array($this->product->getQty());
        }

        foreach ($result as $attrKey => $attrValues) {
            if (!$attrValues) {
                continue;
            }

            $attributeNode = $attributesNode->addChild('attribute');
            $this->appendCData($attributeNode->addChild('key'), $attrKey);
            $valuesNode = $attributeNode->addChild('values');
            foreach ($attrValues as $value) {
                if (is_array($value)) {
                    foreach ($value as $subValue) {
                        if (strcasecmp($subValue, 'yes') == 0) {
                            $subValue = 1;
                        } elseif (strcasecmp($subValue, 'no') == 0) {
                            $subValue = 0;
                        }
                        $this->appendCData($valuesNode->addChild('value'), $subValue);
                    }
                } else {
                    if (strcasecmp($value, 'yes') == 0) {
                        $value = 1;
                    } elseif (strcasecmp($value, 'no') == 0) {
                        $value = 0;
                    }
                    $this->appendCData($valuesNode->addChild('value'), $value);
                }
            }
        }
    }

    /**
     * @param array $result
     * @param Product $product
     */
    protected function addAttributesForProduct(&$result, Product $product)
    {
        /** @var $attribute \Magento\Catalog\Model\Entity\Attribute */
        $attributes = $product->getAttributes();
        foreach ($attributes as $attribute) {
            if ($attribute->getIsFilterable() || $attribute->getIsFilterableInSearch()) {
                $code = $attribute->getAttributeCode();
                $value = $product->getAttributeText($code);
                if (!array_key_exists($code, $result)) {
                    $result[$code] = array();
                }

                if ($value && array_search($value, $result[$code]) === false) {
                    $result[$code][] = $value;
                }
            }
        }
    }

    /**
     * Renders all keywords (meta keywords).
     *
     * @param \SimpleXMLElement $allKeywords
     */
    public function renderKeywords(\SimpleXMLElement $allKeywords)
    {
        $metaKeywords = $this->product->getMetaKeyword();
        if ($metaKeywords) {
            $metaKeywords = explode(',', $metaKeywords);
            $keywords = $allKeywords->addChild('keywords');

            if (count($metaKeywords)) {
                foreach ($metaKeywords as $keyword) {
                    $keyword = trim($keyword);
                    if ($keyword !== '') {
                        $this->appendCData($keywords->addChild('keyword'), $keyword);
                    }
                }
            } else {
                $keywords->addChild('keyword');
            }
        }
    }

    /**
     * Renders date added.
     *
     * @param \SimpleXMLElement $dateAdded
     */
    public function renderDateAdded(\SimpleXMLElement $dateAdded)
    {
        $format = 'Y-m-d H:i:s';
        $date = \DateTime::createFromFormat($format, $this->product->getCreatedAt());

        $this->appendCData($dateAdded->addChild('dateAdded'), $date->format(DATE_ATOM));
    }

    /**
     * Render sales frequencies.
     *
     * @param \SimpleXMLElement $salesFrequencies
     */
    public function renderSalesFrequency(\SimpleXMLElement $salesFrequencies)
    {
        $prefix = (string)$this->deploymentConfig->get(ConfigOptionsListConstants::CONFIG_PATH_DB_PREFIX);

        $om=\Magento\Framework\App\ObjectManager::getInstance();
        $ordersCollection = $om->create('Magento\Sales\Model\ResourceModel\Order\Item\Collection')
            ->addAttributeToFilter('product_id', (int)$this->product->getId());
        $ordersCollection->getSelect()
            ->joinLeft(
                array('sfo' => $prefix . 'sales_order'),
                'order_id=' . 'sfo.entity_id',
                array('sfo.customer_group_id', 'sfo.increment_id')
            )
            ->join(
                array('cg' => $prefix . 'customer_group'),
                'sfo.customer_group_id=' . 'cg.customer_group_id',
                array('cg.customer_group_code')
            )
            ->where('sfo.customer_group_id > 0');

        $ordersCollection = $ordersCollection->load();

        $salesFrequenciesCountPerGroup = array();
        $userGroupArr = array();
        $salesFrequenciesCount = 0;

        /** @var \Findologic\Search\Helper\TrackingScripts $helper */
        $helper = $om->get('\Findologic\Search\Helper\TrackingScripts');

        /** @var $order \Magento\Customer\Model\Group */
        foreach ($ordersCollection as $order) {
            $userGroup = $order->getCustomerGroupCode();
            $userGroupHash = $helper->userGroupToHash($this->shopKey, $userGroup);

            $num = array_search($userGroupHash, $userGroupArr);
            if ($num !== false) {
                $salesFrequenciesCountPerGroup[$userGroupHash] += 1;
            } else {
                $userGroupArr[] = $userGroupHash;
                $salesFrequenciesCountPerGroup[$userGroupHash] = 1;
            }

            $salesFrequenciesCount++;
        }

        $ik = ObjectManager::getInstance();
        /** @var  $orders \Magento\Sales\Model\ResourceModel\Order\Item\Collection */
        $orders = $ik->create('Magento\Sales\Model\ResourceModel\Order\Item\Collection')
            ->addAttributeToFilter('product_id', (int)$this->product->getId());

        $orders->getSelect()
            ->join(
                array('sfo' => $prefix . 'sales_order'),
                'order_id=' . 'sfo.entity_id',
                array('sfo.customer_group_id', 'sfo.increment_id')
            )
            ->joinLeft(
                array('cg' => $prefix . 'customer_group'),
                'sfo.customer_group_id=' . 'cg.customer_group_id',
                array('cg.customer_group_code')
            )
            ->where('cg.customer_group_code is null OR sfo.customer_group_id = 0');

        $salesFrequenciesCount += $orders->getSize();

        if ($salesFrequenciesCount > 0) {
            $this->appendCData($salesFrequencies->addChild('salesFrequency'), $salesFrequenciesCount);
        }

        if (count($salesFrequenciesCountPerGroup) > 0) {
            foreach ($salesFrequenciesCountPerGroup as $key => $value) {
                $salesNode = $salesFrequencies->addChild('salesFrequency');
                $this->appendCData($salesNode, $value);
                $salesNode->addAttribute('usergroup', $key);
            }
        }
    }

    /**
     * Renders all properties.
     *
     * @param \SimpleXMLElement $allProperties
     */
    public function renderAllProperties(\SimpleXMLElement $allProperties)
    {
        $properties = $allProperties->addChild('properties');
        $hasVariants = 0;
        $showFromPrice = 0;

        // Calculate quantity for product.
        $stockQty = ($this->product->getStockItem() !== null) ? (int)$this->product->getStockItem()->getQty() : 0;
        $quantity = $stockQty + (int)$this->product->getQty();

        foreach ($this->variations as $variation) {
            /** @var  $variation \Magento\Catalog\Model\Product  */
            $stockQty  = ($variation->getStockItem() !== null) ? (int)$variation->getStockItem()->getQty() : 0;
            $quantity += $stockQty + (int)$variation->getQty();

            $hasVariants = 1;

            if (($showFromPrice === 0 && $variation->getPrice()) !== $this->productPrice->actualPrice) {
                $showFromPrice = 1;
            }
        }

        $tax = $this->calculateStoreDefaultTax($this->product->getData('tax_class_id'));

        $productProperties = [
            'base price' => sprintf('%.2f', $this->productPrice->basePrice),
            'special price' => sprintf('%.2f', $this->productPrice->specialPrice),
            'quantity' => $quantity,
            'weight' => $this->product->getWeight(),
            'width' => $this->product->getData('width'),
            'height' => $this->product->getData('height'),
            'length' => $this->product->getData('length'),
            'uvp' => $this->product->getData('msrp'),
            'availability' => $this->product->isInStock(),
            'tax' => $tax,
            'has_variants' => $hasVariants
        ];

        if ($productProperties['has_variants'] == 1) {
            $productProperties['show_from_price'] = $showFromPrice;
        }

        $productProperties = $this->addCustomProperties($productProperties);
        
        foreach ($productProperties as $key => $propertyValue) {
            if (!is_null($propertyValue) && !empty($propertyValue)) {
                $property = $properties->addChild('property');
                $this->appendCData($property->addChild('key'), $key);

                if (is_array($propertyValue)) {
                    $p = $property->addChild('values');

                    foreach ($propertyValue as $value) {
                        $this->appendCData($p->addChild('value'), $value);
                    }

                } else {
                    $this->appendCData($property->addChild('value'), $propertyValue);
                }
            }
        }
    }

    /**
     * Adds custom properties. To be used in child class.
     *
     * @param array $productProperties
     * @return array
     */
    protected function addCustomProperties($productProperties)
    {
        return $productProperties;
    }

    /**
     * @param \SimpleXMLElement $node
     * @param $text
     */
    protected function appendCData(\SimpleXMLElement $node, $text)
    {
        /** @var \DOMElement $domNode */
        $domNode = dom_import_simplexml($node);
        $domNode->appendChild($domNode->ownerDocument->createCDATASection($this->utf8Replace($text)));
    }

    /**
     * @param $text
     * @return mixed
     */
    protected function utf8Replace($text)
    {
        return preg_replace('/[\x00-\x08\x0B\x0C\x0E-\x1F\x80-\x9F]/u', '', trim($text));
    }

    /**
     * Calculate store default tax using store default country.
     *
     * @param $productTaxClassID
     * @return mixed
     */
    protected function calculateStoreDefaultTax($productTaxClassID)
    {
        $om = ObjectManager::getInstance();

        /** @var \Magento\Tax\Model\Calculation $calculationTool */
        $calculationTool = $om->get('Magento\Tax\Model\Calculation');
        $addressRequestObject = $calculationTool->getRateRequest(false, false, false, $this->storeId);
        $addressRequestObject->setProductClassId($productTaxClassID);
        return $calculationTool->getRate($addressRequestObject);
    }

    /**
     * @param \SimpleXMLElement $node
     * @param $relatedNode
     */
    protected function removeEmptyNode(\SimpleXMLElement $node, $relatedNode)
    {
        if (!$node->children()->asXML()) {
            $dom = dom_import_simplexml($relatedNode);
            $dom->parentNode->removeChild($dom);
        }
    }

    /**
     * Generate image URL for the given image file name.
     * Append image URL to images in XML export.
     *
     * @param \SimpleXMLElement $images
     * @param string $imageName
     */
    private function generateExportImage(\SimpleXMLElement $images, $imageName)
    {
        $productImageUrl = $this->getProductCatalogUrl() . $imageName;

        $this->appendCData($images->addChild('image'), $productImageUrl);
    }

    /**
     * Retrieves url for product catalog.
     *
     * @return string
     */
    private function getProductCatalogUrl()
    {
        return $this->getMediaUrl() . 'catalog/product';
    }

    /**
     * Obtains media url for the shop.
     *
     * @return string
     */
    private function getMediaUrl()
    {
        return $this->_urlBuilder->getBaseUrl(['_type' => UrlInterface::URL_TYPE_MEDIA]);
    }
}
