<?php

namespace Findologic\Search\Model;

use Magento\Framework\App\Config\Value;
use Magento\Framework\App\ObjectManager;
use Magento\Store\Model\ScopeInterface;
use Magento\Framework\Exception\LocalizedException;

class ShopkeyValidation extends Value
{
    /**
     * Process additional data before save config
     *
     * @return $this
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function beforeSave()
    {
        $shopKey = trim($this->getValue());
        if ($shopKey) {
            $om = ObjectManager::getInstance();
            
            // Get id of currently edited shop.
            $request = $om->get('\Magento\Framework\App\Request\Http');
            $currentId = $request->getParam('store');
            
            // Get all stores from site. 
            $storeManager = $om->get('\Magento\Store\Model\StoreManagerInterface');
            $stores = $storeManager->getStores();

            $scopeConfig = $om->get('\Magento\Framework\App\Config\ScopeConfigInterface');
            
            // CurrentId is set only if admin editing store's data, otherwise he is editing
            // main website or default configuration.
            // Uncomment this condition if you want to enable main website or default configuration
            // to have same shop key as some store.
            // if (isset($currentId)) {
                // Check if any store has same shopkey and throws exception.
                foreach ($stores as $store) {
                    $storeId = $store->getId();

                    // Skip check for current store, we only need to check for other stores.
                    if (isset($currentId) && $currentId === $storeId) {
                        continue;
                    }

                    $keyValue = $scopeConfig->getValue('findologic/findologic_group/shopkey',
                            ScopeInterface::SCOPE_STORE, $storeId);

                    if ($keyValue === $shopKey) {
                        throw new LocalizedException(__('Shop key already exists! Each store view must have its own shop key.'));
                    }
                }
            //}
            
            // Check if shopkey is in valid format.
            if (!preg_match('/^[A-Z0-9]{32}$/', $shopKey)) {
                throw new LocalizedException(__('Shop key format is not valid!'));
            }
            
            $this->setValue(trim($shopKey));
        }

        return $this;
    }

}
